#include <iostream>
#include <vector>
#include <cstdlib>
#include <unistd.h>
#include <signal.h>

#include "RtMidi.h"

using namespace std;

bool done;

static void finish(int ignore) { done = 1;}

int main()
{
    RtMidiIn *midiin = 0;
    int portNumber = 1;
    vector<unsigned char> message;
    double stamp;
    int nBytes;

    try {
        midiin = new RtMidiIn();
    } catch (RtMidiError &error) {
        error.printMessage();
        return -1;
    }

    midiin->openPort(portNumber);

    cout << "Reading MIDI from port : " << midiin->getPortName(portNumber) << endl;

    done = 0;
    (void) signal(SIGINT, finish);

    while(!done) {
        try {
            do {
                stamp = midiin->getMessage(&message);
                nBytes = message.size();
                if(nBytes != 0) {
                    if ((message[0] & 0xF0) == 0x90) {// Check for note on messages
                        cout << "Note on: " << (unsigned int) (message[0] & 0x0f) + 1
                                << "   " << (unsigned int) message[1] << "  "
                                << (unsigned int) message[2] << endl;
                    }
                    if ((message[0] & 0xF0) == 0x80) {// Check for note off messages
                        cout << "Note off: " << (unsigned int) (message[0] & 0x0f) + 1
                                << "   " <<  (unsigned int) message[1] << "  "
                                << (unsigned int) message[2] << endl;
                    }
                    if ((message[0] & 0xF0) == 0xb0) {// Check for control messages
                        cout << "Control: " << (unsigned int) (message[0] & 0x0f) + 1
                                << "   " <<  (unsigned int) message[1] << "  "
                                << (unsigned int) message[2] << endl;
                    }
                    if ((message[0] & 0xF0) == 0xe0) {// Check for pitch bend
                        int pb = ((message[2] << 7) | message[1] ) - 8192;
                        cout << "Pitch bend: " << (unsigned int) (message[0] & 0x0f) + 1
                                << "   " << pb << endl;
                    }
                }
            } while (nBytes != 0);

        } catch (RtMidiError &error) {
            error.printMessage();
            //goto cleanup;
        }

        usleep(10000);
    }


cleanup:
    midiin->closePort();
    delete midiin;
    cout << "Done." << endl;

    return 0;
}

